@@ -194,7 +194,7 @@ static char *argstr_get_word (const char **argstr)
 		/* Copy any accumulated literal characters. */
 		if (litstart < *argstr) {
 			char *tmp = xstrndup (litstart, *argstr - litstart);
-			out = appendstr (out, tmp, NULL);
+			out = appendstr (out, tmp, (void *) 0);
 			free (tmp);
 		}
 
@@ -231,7 +231,7 @@ static char *argstr_get_word (const char **argstr)
 					return NULL;
 				}
 				backslashed[1] = '\0';
-				out = appendstr (out, backslashed, NULL);
+				out = appendstr (out, backslashed, (void *) 0);
 				litstart = ++*argstr;
 				break;
 
@@ -249,7 +249,7 @@ static char *argstr_get_word (const char **argstr)
 	/* Copy any accumulated literal characters. */
 	if (litstart < *argstr) {
 		char *tmp = xstrndup (litstart, *argstr - litstart);
-		out = appendstr (out, tmp, NULL);
+		out = appendstr (out, tmp, (void *) 0);
 		free (tmp);
 	}
 
@@ -371,7 +371,7 @@ pipecmd *pipecmd_new_sequence (const char *name, ...)
 	return cmd;
 }
 
-static void passthrough (void *data PIPELINE_ATTR_UNUSED)
+static void passthrough (void *data _GL_UNUSED)
 {
 	for (;;) {
 		char buffer[4096];
@@ -535,7 +535,7 @@ void pipecmd_argstr (pipecmd *cmd, const char *argstr)
 	}
 }
 
-int pipecmd_get_nargs (pipecmd *cmd)
+int _GL_ATTRIBUTE_PURE pipecmd_get_nargs (pipecmd *cmd)
 {
 	struct pipecmd_process *cmdp;
 
@@ -695,58 +695,60 @@ char *pipecmd_tostring (pipecmd *cmd)
 
 	if (cmd->cwd_fd >= 0) {
 		char *cwd_fd_str = xasprintf ("%d", cmd->cwd_fd);
-		out = appendstr (out, "(cd <fd ", cwd_fd_str, "> && ", NULL);
+		out = appendstr (out, "(cd <fd ", cwd_fd_str, "> && ",
+				 (void *) 0);
 		free (cwd_fd_str);
 	} else if (cmd->cwd)
-		out = appendstr (out, "(cd ", cmd->cwd, " && ", NULL);
+		out = appendstr (out, "(cd ", cmd->cwd, " && ", (void *) 0);
 
 	for (i = 0; i < cmd->nenv; ++i) {
 		if (cmd->env[i].name)
 			out = appendstr (out, cmd->env[i].name, "=",
 					 cmd->env[i].value ? cmd->env[i].value
 							   : "<unset>",
-					 " ", NULL);
+					 " ", (void *) 0);
 		else
-			out = appendstr (out, "env -i ", NULL);
+			out = appendstr (out, "env -i ", (void *) 0);
 	}
 
 	switch (cmd->tag) {
 		case PIPECMD_PROCESS: {
 			struct pipecmd_process *cmdp = &cmd->u.process;
 
-			out = appendstr (out, cmd->name, NULL);
+			out = appendstr (out, cmd->name, (void *) 0);
 			for (i = 1; i < cmdp->argc; ++i)
 				/* TODO: escape_shell()? */
 				out = appendstr (out, " ", cmdp->argv[i],
-						 NULL);
+						 (void *) 0);
 
 			break;
 		}
 
 		case PIPECMD_FUNCTION:
-			out = appendstr (out, cmd->name, NULL);
+			out = appendstr (out, cmd->name, (void *) 0);
 			break;
 
 		case PIPECMD_SEQUENCE: {
 			struct pipecmd_sequence *cmds = &cmd->u.sequence;
 
-			out = appendstr (out, "(", NULL);
+			out = appendstr (out, "(", (void *) 0);
 			for (i = 0; i < cmds->ncommands; ++i) {
 				char *subout = pipecmd_tostring
 					(cmds->commands[i]);
-				out = appendstr (out, subout, NULL);
+				out = appendstr (out, subout, (void *) 0);
 				free (subout);
 				if (i < cmds->ncommands - 1)
-					out = appendstr (out, " && ", NULL);
+					out = appendstr (out, " && ",
+							 (void *) 0);
 			}
-			out = appendstr (out, ")", NULL);
+			out = appendstr (out, ")", (void *) 0);
 
 			break;
 		}
 	}
 
 	if (cmd->cwd_fd >= 0 || cmd->cwd)
-		out = appendstr (out, ")", NULL);
+		out = appendstr (out, ")", (void *) 0);
 
 	return out;
 }
@@ -818,7 +820,8 @@ void pipecmd_exec (pipecmd *cmd)
 				cmdf->free_func (cmdf->data);
 			if (cmd->pre_exec_free_func)
 				cmd->pre_exec_free_func (cmd->pre_exec_data);
-			exit (0);
+			fflush (NULL);
+			_exit (0);
 		}
 
 		case PIPECMD_SEQUENCE: {
@@ -895,12 +898,12 @@ void pipecmd_exec (pipecmd *cmd)
 
 				if (WIFSIGNALED (status)) {
 					raise (WTERMSIG (status));
-					exit (1); /* just to make sure */
+					_exit (1); /* just to make sure */
 				} else if (status && WIFEXITED (status))
-					exit (WEXITSTATUS (status));
+					_exit (WEXITSTATUS (status));
 			}
 
-			exit (0);
+			_exit (0);
 		}
 	}
 
@@ -908,7 +911,7 @@ void pipecmd_exec (pipecmd *cmd)
 	/* Never called, but gcc doesn't realise that error with non-zero
 	 * status always exits.
 	 */
-	exit (EXEC_FAILED_EXIT_STATUS);
+	_exit (EXEC_FAILED_EXIT_STATUS);
 }
 
 void pipecmd_free (pipecmd *cmd)
@@ -1151,12 +1154,12 @@ void pipeline_commands (pipeline *p, ...)
 	va_end (cmdv);
 }
 
-int pipeline_get_ncommands (pipeline *p)
+int _GL_ATTRIBUTE_PURE pipeline_get_ncommands (pipeline *p)
 {
 	return p->ncommands;
 }
 
-pipecmd *pipeline_get_command (pipeline *p, int n)
+pipecmd * _GL_ATTRIBUTE_PURE pipeline_get_command (pipeline *p, int n)
 {
 	if (n < 0 || n >= p->ncommands)
 		return NULL;
@@ -1173,7 +1176,7 @@ pipecmd *pipeline_set_command (pipeline *p, int n, pipecmd *cmd)
 	return prev;
 }
 
-pid_t pipeline_get_pid (pipeline *p, int n)
+pid_t _GL_ATTRIBUTE_PURE pipeline_get_pid (pipeline *p, int n)
 {
 	assert (p->pids);	/* pipeline started */
 	if (n < 0 || n >= p->ncommands)
@@ -1261,10 +1264,10 @@ char *pipeline_tostring (pipeline *p)
 
 	for (i = 0; i < p->ncommands; ++i) {
 		char *cmdout = pipecmd_tostring (p->commands[i]);
-		out = appendstr (out, cmdout, NULL);
+		out = appendstr (out, cmdout, (void *) 0);
 		free (cmdout);
 		if (i < p->ncommands - 1)
-			out = appendstr (out, " | ", NULL);
+			out = appendstr (out, " | ", (void *) 0);
 	}
 
 	return out;
@@ -2221,7 +2224,7 @@ const char *pipeline_peek (pipeline *p, size_t *len)
 	return get_block (p, len, 1);
 }
 
-size_t pipeline_peek_size (pipeline *p)
+size_t _GL_ATTRIBUTE_PURE pipeline_peek_size (pipeline *p)
 {
 	if (!p->buffer)
 		return 0;
